package com.elab.core.kafka.consumer;

import com.alibaba.fastjson.JSON;
import com.elab.log.utils.CatCrossProcess;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.kafka.listener.AcknowledgingMessageListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;

/**
 * @Module TODO
 * @Description TODO
 * @Author liukaixiong
 * @Date 2020/10/27 19:14
 */
public class KafkaMsgListener implements AcknowledgingMessageListener<String, String>, InitializingBean {
    private Logger logger = LoggerFactory.getLogger(getClass());
    private List<AbstractKafkaConsumer> messageListeners;

    public KafkaMsgListener(List<AbstractKafkaConsumer> messageListeners) {
        Assert.notNull(messageListeners, "请实现一个AbstractKafkaConsumer的子类,并注册到ioc容器中");
        this.messageListeners = messageListeners;
    }

    @Override
    public void onMessage(ConsumerRecord<String, String> data, Acknowledgment acknowledgment) {
        Map<String, String> valueMap = JSON.parseObject(data.value(), Map.class);
        CatCrossProcess.buildRemoteMQMsg("kafka-" + data.topic(), data.partition() + "", valueMap, () -> {
            try {
                String topic = data.topic();
                this.messageListeners.forEach((listener) -> {
                    if (listener.subscribeTopic().equals(topic)) {
                        listener.onMessage(data, acknowledgment);
                    }
                });
            } catch (Exception e) {
                throw e;
            }
            return acknowledgment;
        });
    }


    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
